#include <bits/stdc++.h>
using namespace std;

void ensureImpl(bool a, const char *cond, int line)
{
    if(!a)
    {
        fprintf(stderr, "Line %d: failed condition %s\n", line, cond);
        asm volatile("int3");
        exit(1);
    }
}
#define ensure(x) ensureImpl(x, #x, __LINE__)

enum ValueType
{
    UNDEFINED,
    INT,
    STRING,
};
struct Value
{
    ValueType type;
    int dataInt;
    string dataString;
    Value()
    {
        type = UNDEFINED;
    }
    int cmp(const Value &other) const
    {
        ensure(type != UNDEFINED);
        ensure(type == other.type);
        if(type == INT) return dataInt - other.dataInt;
        else return strcmp(dataString.c_str(), other.dataString.c_str());
    }
    void print() const
    {
        ensure(type != UNDEFINED);
        if(type == INT) cout << dataInt;
        else cout << dataString;
    }
    bool operator ==(const Value &other) const
    {
        return cmp(other) == 0;
    }
    bool operator !=(const Value &other) const
    {
        return cmp(other) != 0;
    }
    bool operator <(const Value &other) const
    {
        return cmp(other) < 0;
    }
    bool operator <=(const Value &other) const
    {
        return cmp(other) <= 0;
    }
    bool operator >(const Value &other) const
    {
        return cmp(other) > 0;
    }
    bool operator >=(const Value &other) const
    {
        return cmp(other) >= 0;
    }
};
namespace std
{
    template<> struct hash<Value>
    {
        hash<string> stringHash;
        hash<int> intHash;
        size_t operator()(const Value &value) const
        {
            ensure(value.type != UNDEFINED);
            if(value.type == INT) return intHash(value.dataInt);
            else return stringHash(value.dataString);
        }
    };
}
bool isIntField(const string &fieldName)
{
    return fieldName == "age" || fieldName == "score";
}
Value readValue(const string &fieldName, istream &is = cin)
{
    Value ans;
    if(isIntField(fieldName))
    {
        ans.type = INT;
        is >> ans.dataInt;
    }
    else
    {
        ans.type = STRING;
        is >> ans.dataString;
    }
    return ans;
}
Value valueFromString(const string &fieldName, const string &s)
{
    stringstream ss(s);
    Value ans = readValue(fieldName, ss);
    if(ans.type == STRING)
    {
        int n = ans.dataString.length();
        ensure(n >= 2);
        ensure(ans.dataString[0] == '"');
        ensure(ans.dataString[n - 1] == '"');
        ans.dataString = ans.dataString.substr(1, n - 2);
    }
    return ans;
}

struct Record
{
    map<string, Value> values;
};
Record readRecord(const vector<string> &fieldNames)
{
    Record ans;
    for(const string &field: fieldNames) ans.values.emplace(field, readValue(field));
    return ans;
}

struct Table
{
    vector<Record> records;
};
Table readTable(const vector<string> &fieldNames)
{
    Table ans;
    int n; cin >> n;
    while(n--) ans.records.emplace_back(readRecord(fieldNames));
    return ans;
}

struct DB
{
    map<string, Table> tables;
};
map<string, vector<string>> schema;
struct SchemaInitializer
{
    SchemaInitializer()
    {
        schema.emplace("Student", vector<string>({"sid", "dept", "age"}));
        schema.emplace("Course", vector<string>({"cid", "name"}));
        schema.emplace("Teacher", vector<string>({"tid", "dept", "age"}));
        schema.emplace("Grade", vector<string>({"sid", "cid", "score"}));
        schema.emplace("Teach", vector<string>({"cid", "tid"}));
    }
} schemaInitializer;
DB readDB()
{
    DB ans;
    ans.tables.emplace("Student", readTable({"sid", "dept", "age"}));
    ans.tables.emplace("Course", readTable({"cid", "name"}));
    ans.tables.emplace("Teacher", readTable({"tid", "dept", "age"}));
    ans.tables.emplace("Grade", readTable({"sid", "cid", "score"}));
    ans.tables.emplace("Teach", readTable({"cid", "tid"}));
    return ans;
}
vector<string> tokenize(string s)
{
    s += ' ';
    vector<string> ans; string t;
    for(char c: s) if(strchr(",.>=< ", c))
    {
        if(t.length())
        {
            ans.emplace_back(t);
            t.clear();
        }
        if(c != ' ') ans.emplace_back(1, c);
    }
    else t += c;
    return ans;
}

struct ConstantCondition
{
    string tableName, fieldName;
    char cmp; // < = >
    Value target;
};
struct ColumnCondition
{
    string tableName1, fieldName1, tableName2, fieldName2;
};
struct WhereConditions
{
    ColumnCondition columnCondition;
    vector<ConstantCondition> constantConditions;
    bool hasColumnCondition()
    {
        return columnCondition.tableName1.size();
    }
    bool earlyCheck(Record &record, const string &tableName)
    {
        for(const ConstantCondition &constantCondition: constantConditions)
        {
            if(tableName == constantCondition.tableName)
            {
                const Value &v = record.values[constantCondition.fieldName];
                if(constantCondition.cmp == '<')
                {
                    if(v >= constantCondition.target) return false;
                }
                else if(constantCondition.cmp == '=')
                {
                    if(v != constantCondition.target) return false;
                }
                else if(constantCondition.cmp == '>')
                {
                    if(v <= constantCondition.target) return false;
                }
                else ensure(false);
            }
        }
        return true;
    }
    bool check(Record &record)
    {
        if(hasColumnCondition())
        {
            const Value &v1 = record.values[columnCondition.tableName1 + "." + columnCondition.fieldName1];
            const Value &v2 = record.values[columnCondition.tableName2 + "." + columnCondition.fieldName2];
            if(v1 != v2) return false;
        }
        for(const ConstantCondition &constantCondition: constantConditions)
        {
            const Value &v = record.values[constantCondition.tableName + "." + constantCondition.fieldName];
            if(constantCondition.cmp == '<')
            {
                if(v >= constantCondition.target) return false;
            }
            else if(constantCondition.cmp == '=')
            {
                if(v != constantCondition.target) return false;
            }
            else if(constantCondition.cmp == '>')
            {
                if(v <= constantCondition.target) return false;
            }
            else ensure(false);
        }
        return true;
    }
};

bool isColumnCondition(string *begin, string *end)
{
    int cnt = 0;
    for(string *i = begin; i != end; ++i) cnt += (*i == ".");
    ensure(cnt <= 2);
    return cnt == 2;
}
void parseOneWhereCondition(string *begin, string *end, WhereConditions &ans)
{
    if(isColumnCondition(begin, end))
    {
        ensure(!ans.hasColumnCondition());
        // tableName1 . fieldName1 = tableName2 . fieldName2
        ensure(end - begin == 7);
        ans.columnCondition.tableName1 = begin[0];
        ensure(begin[1] == ".");
        ans.columnCondition.fieldName1 = begin[2];
        ensure(begin[3] == "=");
        ans.columnCondition.tableName2 = begin[4];
        ensure(begin[5] == ".");
        ans.columnCondition.fieldName2 = begin[6];
        ensure(begin[0] != begin[4]);
        ensure(begin[2] == begin[6]);
    }
    else if(end - begin == 5)
    {
        // tableName . fieldName cmp target
        ConstantCondition constantCondition;
        constantCondition.tableName = begin[0];
        ensure(begin[1] == ".");
        constantCondition.fieldName = begin[2];
        ensure(begin[3].size() == 1);
        ensure(strchr(">=<", begin[3][0]));
        constantCondition.cmp = begin[3][0];
        constantCondition.target = valueFromString(begin[2], begin[4]);
        ans.constantConditions.emplace_back(constantCondition);
    }
    else
    {
        ensure(end - begin == 3);
        // fieldName cmp constant
        ConstantCondition constantCondition;
        constantCondition.fieldName = begin[0];
        ensure(begin[1].size() == 1);
        ensure(strchr(">=<", begin[1][0]));
        constantCondition.cmp = begin[1][0];
        constantCondition.target = valueFromString(begin[0], begin[2]);
        ans.constantConditions.emplace_back(constantCondition);
    }
}
WhereConditions parseWhere(string *where, string *end)
{
    WhereConditions ans;
    while(where != end)
    {
        string *AND = find(where + 1, end, "AND");
        parseOneWhereCondition(where + 1, AND, ans);
        where = AND;
    }
    return ans;
}

vector<string> parseFrom(string *from, string *end)
{
    vector<string> ans;
    for(string *i = from + 1; i != end; ++i) if(*i != ",") ans.emplace_back(*i);
    return ans;
}

struct SelectColumn
{
    string tableName, fieldName;
};
SelectColumn parseOneSelect(string *begin, string *end)
{
    SelectColumn ans;
    if(end - begin == 3)
    {
        // tableName . fieldName
        ans.tableName = begin[0];
        ensure(begin[1] == ".");
        ans.fieldName = begin[2];
    }
    else
    {
        ensure(end - begin == 1);
        ans.fieldName = begin[0];
    }
    return ans;
}
vector<SelectColumn> parseSelect(string *select, string *end)
{
    vector<SelectColumn> ans;
    while(select != end)
    {
        string *comma = find(select + 1, end, ",");
        ans.emplace_back(parseOneSelect(select + 1, comma));
        select = comma;
    }
    return ans;
}

Table joinOneTable(DB &db, const string &table1)
{
    Table ans;
    for(Record &record1: db.tables[table1].records)
    {
        Record record;
        for(const pair<string, Value> &value: record1.values)
        {
            record.values.emplace(table1 + "." + value.first, value.second);
        }
        ans.records.emplace_back(record);
    }
    return ans;
}
Table joinTwoTablesNoCondition(DB &db, const string &table1, const Table &t1, const string &table2, const Table &t2)
{
    Table ans;
    for(const Record &record1: t1.records)
    {
        for(const Record &record2: t2.records)
        {
            Record record;
            for(const pair<string, Value> &value: record1.values)
            {
                record.values.emplace(table1 + "." + value.first, value.second);
            }
            for(const pair<string, Value> &value: record2.values)
            {
                record.values.emplace(table2 + "." + value.first, value.second);
            }
            ans.records.emplace_back(record);
        }
    }
    return ans;
}
Table joinTwoTablesWithCondition(DB &db, const ColumnCondition &columnCondition, const string &table1, Table &t1, const string &table2, Table &t2)
{
    if(columnCondition.tableName1 == table2)
    {
        return joinTwoTablesWithCondition(db, columnCondition, table2, t2, table1, t1);
    }
    ensure(columnCondition.tableName1 == table1);
    unordered_map<Value, vector<Record>> lookupTable;
    for(Record &record2: t2.records)
    {
        lookupTable[record2.values[columnCondition.fieldName2]].emplace_back(record2);
    }
    Table ans;
    for(Record &record1: t1.records)
    {
        for(const Record &record2: lookupTable[record1.values[columnCondition.fieldName1]])
        {
            Record record;
            for(const pair<string, Value> &value: record1.values)
            {
                record.values.emplace(table1 + "." + value.first, value.second);
            }
            for(const pair<string, Value> &value: record2.values)
            {
                record.values.emplace(table2 + "." + value.first, value.second);
            }
            ans.records.emplace_back(record);
        }
    }
    return ans;
}

void handleQuery(DB &db, string *begin, string *end)
{
    string *select = find(begin, end, "SELECT");
    string *from = find(begin, end, "FROM");
    string *where = find(begin, end, "WHERE");
    ensure(select != end);
    ensure(from != end);

    WhereConditions whereConditions = parseWhere(where, end);
    vector<string> fromTables = parseFrom(from, where);
    vector<SelectColumn> selectColumns = parseSelect(select, from);

    Table joinedTable;

    ensure(fromTables.size() <= 2);
    ensure(fromTables.size() >= 1);
    for(const string &tableName: fromTables)
    {
        set<string> currentSchema(schema[tableName].begin(), schema[tableName].end());
        for(ConstantCondition &constantCondition: whereConditions.constantConditions)
        {
            if(constantCondition.tableName.empty() && currentSchema.count(constantCondition.fieldName))
            {
                constantCondition.tableName = tableName;
            }
        }
        for(SelectColumn &selectColumn: selectColumns)
        {
            if(selectColumn.tableName.empty() && currentSchema.count(selectColumn.fieldName))
            {
                selectColumn.tableName = tableName;
            }
        }
    }

    if(fromTables.size() == 1)
    {
        joinedTable = joinOneTable(db, fromTables[0]);
    }
    else
    {
        Table t1, t2;
        for(Record &record: db.tables[fromTables[0]].records)
        {
            if(whereConditions.earlyCheck(record, fromTables[0]))
            {
                t1.records.emplace_back(record);
            }
        }
        for(Record &record: db.tables[fromTables[1]].records)
        {
            if(whereConditions.earlyCheck(record, fromTables[1]))
            {
                t2.records.emplace_back(record);
            }
        }
        if(whereConditions.hasColumnCondition())
        {
            joinedTable = joinTwoTablesNoCondition(db, fromTables[0], t1, fromTables[1], t2);
            // joinedTable = joinTwoTablesWithCondition(db, whereConditions.columnCondition, fromTables[0], t1, fromTables[1], t2);
        }
        else
        {
            joinedTable = joinTwoTablesNoCondition(db, fromTables[0], t1, fromTables[1], t2);
        }
    }

    ensure(selectColumns.size() >= 1);
    if(selectColumns[0].fieldName == "*")
    {
        selectColumns.clear();
        for(const string &table: fromTables)
        {
            for(const string &field: schema[table])
            {
                SelectColumn selectColumn;
                selectColumn.tableName = table;
                selectColumn.fieldName = field;
                selectColumns.emplace_back(move(selectColumn));
            }
        }
    }
    for(Record &record: joinedTable.records)
    {
        if(whereConditions.check(record))
        {
            bool first = true;
            for(const SelectColumn &selectColumn: selectColumns)
            {
                if(first) first = false; else putchar(' ');
                const Value &v = record.values[selectColumn.tableName + "." + selectColumn.fieldName];
                v.print();
            }
            cout << endl;
        }
    }
}

int main()
{
    DB db = readDB();
    int q; cin >> q;
    string s; getline(cin, s);
    while(q--)
    {
        getline(cin, s);
        // cout << s << endl;
        vector<string> tokens = tokenize(s);
        handleQuery(db, tokens.data(), tokens.data() + tokens.size());
    }
    return 0;
}
