#include <iostream>
using namespace std;
#include <stdio.h>
#include <fstream>
#include <string>
#include <cstring>
#include <vector>
#include <map>

#define TABLE_NUM 5
#define MAX_SQL_LEN 100
#define MAX_OUTPUT_SIZE 10000

enum data_type {Int, Str};

struct data{
    int i;
    string s;
    data_type type;

    data(string v)
    {
        i = 0;
        s = v;
        type = Str;
    }
    data(char* v)
    {
        i = 0;
        s = v;
        type = Str;
    }
    data(int v = 0)
    {
        i = v;
        type = Int;
    }

    void print()
    {
        if(type==Int) printf("%d", i);
        else printf("%s", s.c_str());
    }
};

bool operator<(const data & a, const data & b)
{
    if(a.type==Int) return a.i < b.i;
    else return a.s < b.s;
}

struct column_type{
    int table, attr;
};

struct cond_type1{  // column cmp constant
    column_type column;
    data cons;
    int cmp;    // -1:<, 0:=, 1:>
};

struct cond_type2{  // column = column
    column_type column1, column2;
    int cmp;
};

bool format_error(string error_info)
{
    cerr << error_info << endl;
    exit(0);
}

bool cmp(data r1, data r2, char t)
{
    if(r1.type!=r2.type) return format_error("Int cmp Str");
    if(r1.type==Str)
    {
        if(t!='=') return format_error("Str >/< Str");
        else return r1.s==r2.s;
    }
    else{
        switch(t)
        {
        case '=':
            return r1.i == r2.i;
        case '<':
            return r1.i < r2.i;
        case '>':
            return r1.i > r2.i;
        default:
            return format_error("cmp is not in [=, <, >]");
        }
    }
}

map<string, int> table_ID;

struct table_attr{
    int n, m;   // number of rows and columns
    vector<string> name;
    vector<data_type> type;
};
table_attr ta[TABLE_NUM];

typedef vector<data> record;
typedef vector<record> table;
table tables[TABLE_NUM];

void init_table()
{
    table_ID["Student"] = 0;
    table_ID["Course"] = 1;
    table_ID["Teacher"] = 2;
    table_ID["Grade"] = 3;
    table_ID["Teach"] = 4;

    ta[0].type = {Str, Str, Int};
    ta[0].name = {"sid", "dept", "age"};
    ta[0].m = 3;

    ta[1].type = {Str, Str};
    ta[1].name = {"cid", "name"};
    ta[1].m = 2;

    ta[2].type = {Str, Str, Int};
    ta[2].name = {"tid", "dept", "age"};
    ta[2].m = 3;

    ta[3].type = {Str, Str, Int};
    ta[3].name = {"sid", "cid", "score"};
    ta[3].m = 3;

    ta[4].type = {Str, Str};
    ta[4].name = {"cid", "tid"};
    ta[4].m = 2;
}

void read_tables()
{
    for(int i=0; i<TABLE_NUM; i++)
    {
        scanf("%d", &ta[i].n);
        tables[i].resize(ta[i].n);
        for(int j=0; j<ta[i].n; j++)
        {
            for(int k=0; k<ta[i].m; k++)
            {
                if(ta[i].type[k]==Int)
                {
                    int tmp;
                    scanf("%d", &tmp);
                    tables[i][j].push_back(data(tmp));
                }
                else
                {
                    char tmp[20];
                    scanf("%s", tmp);
                    tables[i][j].push_back(data(tmp));
                }
            }
        }
    }
}

bool is_digit(char ch)
{
    if(ch<'0' || ch>'9') return false;
    else return true;
}

bool are_digits(string s)
{
    for(int i=0; i<s.size(); i++)
        if(!is_digit(s[i])) return false;
    return true;
}

int str_to_int(string s)
{
    int tmp = 0;
    for(int i=0; i<s.size(); i++)
    {
        tmp = tmp * 10 + s[i] - '0';
    }
    return tmp;
}

bool is_char(char ch)
{
    if((ch>='a' && ch<='z') || (ch>='A' && ch<='Z')) return true;
    else return false;
}

bool are_chars(string s)
{
    for(int i=0; i<s.size(); i++)
        if(!is_char(s[i])) return false;
    return true;
}

void word_split(char* csql, vector<string> & sql)
{
    int i = 0;
    string tmp;
    bool flag = true; // pair "
    while(true)
    {
        switch(csql[i])
        {
        case '\0':
        case ' ':
            if(!tmp.empty())
            {
                sql.push_back(tmp);
                tmp.clear();
            }
            break;

        case ',':
        case '.':
        case '<':
        case '>':
        case '=':
        case '*':
            if(!tmp.empty())
            {
                sql.push_back(tmp);
                tmp.clear();
            }
            tmp.push_back(csql[i]);
            sql.push_back(tmp);
            tmp.clear();
            break;

        case '\"':
            if(!tmp.empty())
            {
                sql.push_back(tmp);
                tmp.clear();
            }
            tmp.push_back(csql[i]);
            if(flag) tmp.push_back('0');
            else tmp.push_back('1');
            flag = not flag;
            sql.push_back(tmp);
            tmp.clear();
            break;

        default:
            if(is_digit(csql[i]) || is_char(csql[i])) tmp.push_back(csql[i]);
            else format_error(csql+i);
        }
        if(csql[i]=='\0') break;
        else i++;
    }
    if(!flag) format_error("\" not paired");
    //cout << sql.size() << endl;
    //for(int i=0; i<sql.size(); i++)
        //cout << sql[i] << endl;
}

void locate(vector<string> & sql, int & p_select, int & p_from, int & p_where, int & p_END)
{
    p_END = sql.size();

    if(sql[0]!="SELECT") format_error("sql[0] is not SELECT");
    p_from = p_select = 0;

    while(p_from<p_END && sql[p_from]!="FROM") p_from++;
    if(p_from==p_END) format_error("FROM not found");

    p_where = p_from;
    while(p_where<p_END && sql[p_where]!="WHERE") p_where++;
}

void extract_tables(const vector<string> & sql, int p_from, int p_where, int *t)
{
    int len = p_where - p_from - 1;
    if(len==1)  // one table
    {
        if(table_ID.count(sql[p_from+1])==0) format_error("FROM ERROR");
        t[0] = table_ID[sql[p_from+1]];
        t[1] = -1;
    }
    else if(len==3) // two tables
    {
        if(table_ID.count(sql[p_from+1])==0 || sql[p_from+2]!="," || table_ID.count(sql[p_from+3])==0) format_error("FROM ERROR");
        t[0] = table_ID[sql[p_from+1]];
        t[1] = table_ID[sql[p_from+3]];
        if(t[0]==t[1]) format_error("FROM ERROR: two identical tables");
    }
    else format_error("FROM ERROR");
}

column_type get_column(const vector<string> & sql, int pl, int pr, int *t)
{
    column_type tmp;
    int len = pr - pl - 1;
    if(len==1)  // not specify table_name
    {
        int i, j;
        for(i=0; i<2; i++)
        {
            if(t[i]==-1) continue;
            bool find_attr = false;
            for(j=0; j<ta[t[i]].m; j++)
                if(ta[t[i]].name[j]==sql[pl+1])
            {
                find_attr = true;
                break;
            }
            if(find_attr) break;
        }
        if(i==2) format_error("COLUMN ERROR: wrong column_name 1");
        tmp.table = t[i];
        tmp.attr = j;
    }
    else if(len==3) // two tables
    {
        if(table_ID.count(sql[pl+1])==0 || sql[pl+2]!="." ) format_error("COLUMN ERROR");
        int i = table_ID[sql[pl+1]];
        if(i!=t[0] && i!=t[1]) format_error("COLUMN ERROR: wrong table_name");
        int j;
        for(j=0; j<ta[i].m; j++)
            if(ta[i].name[j]==sql[pl+3]) break;
        if(j==ta[i].m) format_error("COLUMN ERROR: wrong column_name 2");
        tmp.table = i;
        tmp.attr = j;
    }
    else format_error("COLUMN ERROR");
    return tmp;
}

void extract_columns(const vector<string> & sql, int p_select, int p_from, int *t, vector<column_type> & columns)
{
    if(p_from-p_select==2 && sql[p_select+1]=="*") return;
    int i = p_select;
    while(i<p_from)
    {
        int j = i + 1;
        while(j<p_from && sql[j]!=",") j++;
        columns.push_back(get_column(sql, i, j, t));
        i = j;
    }
}

bool get_constant(const vector<string> & sql, int pl, int pr, data & cons)
{
    int len = pr - pl - 1;
    if(len==1 && are_digits(sql[pl+1]))
    {
        if(sql[pl+1].size()>3) format_error("integer constant's length>3");
        cons = data(str_to_int(sql[pl+1]));
        if(cons.i<0 || cons.i>100) format_error("integer constant is not in [0, 100]");
        return true;
    }
    else if(len==3 && sql[pl+1]=="\"0" && sql[pl+3]=="\"1")
    {
        if(sql[pl+2].size()>10) format_error("string constant's length>10");
        if(!are_chars(sql[pl+2]) && !are_digits(sql[pl+2])) format_error("string constant error");
        cons = data(sql[pl+2]);
        return true;
    }
    else return false;
}

void extract_cond(const vector<string> & sql, int p_where, int p_END, int *t, vector<cond_type1> & cond_t0, vector<cond_type1> & cond_t1, vector<cond_type2> & cond_tt)
{
    if(p_where==p_END) return;
    cond_type1 tmp1;
    cond_type2 tmp2;
    int i = p_where;
    while(i<p_END)
    {
        int j = i + 1;
        while(j<p_END && sql[j]!="<" && sql[j]!="=" && sql[j]!=">") j++;
        if(j==p_END) format_error("cond format error: </=/> not found");
        int k = j + 1;
        while(k<p_END && (sql[k]!="AND" || sql[k-1]=="\"0")) k++; // make sure it is not a fake AND, e.g. AND name = "AND"
        if(get_constant(sql, j, k, tmp1.cons))
        {
            if(sql[j]=="<") tmp1.cmp = -1;
            else if(sql[j]=="=") tmp1.cmp = 0;
            else tmp1.cmp = 1;
            tmp1.column = get_column(sql, i, j, t);
            if(ta[tmp1.column.table].type[tmp1.column.attr]!=tmp1.cons.type) format_error("column cmp constant error: type is different");
            if(tmp1.cons.type==Str)
            {
                if(tmp1.cmp!=0) format_error("column cmp constant error: Str </> Str");
                string cname = ta[tmp1.column.table].name[tmp1.column.attr];
                if(cname=="cid" || cname=="tid" || cname=="sid")
                {
                    if(!are_digits(sql[j+2])) format_error("column cmp constant error: type is different 1");
                }
                else if(!are_chars(sql[j+2])) format_error("column cmp constant error: type is different a");
            }

            if(tmp1.column.table==t[0]) cond_t0.push_back(tmp1);
            else cond_t1.push_back(tmp1);
        }
        else
        {
            if(sql[j]=="=") tmp2.cmp = 0;
            else format_error("column cmp column error: cmp is not =");
            tmp2.column1 = get_column(sql, i, j, t);
            tmp2.column2 = get_column(sql, j, k, t);
            int t1 = tmp2.column1.table;
            int t2 = tmp2.column2.table;
            if(t1==t2) format_error("column cmp column error: from the same table");
            if(ta[t1].name[tmp2.column1.attr]!=ta[t2].name[tmp2.column2.attr]) format_error("column cmp column error: different column_name");
            cond_tt.push_back(tmp2);
        }
        i = k;
    }
    if(cond_tt.size()>1) format_error("column cmp column error: more than 1");
}

bool check_cond(data a, data b, int cmp)
{
    if(a.type!=b.type) format_error("check_cond error: type is different");
    if(a.type==Int)
    {
        switch(cmp)
        {
        case -1:
            return a.i < b.i;
        case 0:
            return a.i == b.i;
        case 1:
            return a.i > b.i;
        default:
            return format_error("check_cond error: cmp is not in [-1, 0, 1]");
        }
    }
    else{
        if(cmp==0) return a.s == b.s;
        else return format_error("check_cond error: Str </> Str");
    }
}

void select_records(int t, const vector<cond_type1> & cond, vector<int> & records)
{
    if(t==-1) return;
    for(int i=0; i<ta[t].n; i++)
    {
        bool ok = true;
        for(int j=0; j<cond.size(); j++)
        {
            if(!check_cond(tables[t][i][cond[j].column.attr], cond[j].cons, cond[j].cmp))
            {
                ok = false;
                break;
            }
        }
        if(ok) records.push_back(i);
    }
}

void print_all_columns(const vector<pair<int,int>> & records, int * t)
{
    for(int k=0; k<records.size(); k++)
    {
        int r[2];
        r[0] = records[k].first;
        r[1] = records[k].second;
        for(int i=0; i<2; i++)
        {
            if(t[i]==-1) continue;
            for(int j=0; j<ta[t[i]].m; j++)
            {
                if(j>0 || i>0) printf("%c",' ');
                tables[t[i]][r[i]][j].print();
            }
        }
        printf("%c",'\n');
    }
}

void print_columns(const vector<pair<int, int>> & records, int * t, const vector<column_type> & columns)
{
    for(int k=0; k<records.size(); k++)
    {
        int r[2];
        r[0] = records[k].first;
        r[1] = records[k].second;
        for(int i=0; i<columns.size(); i++)
        {
            int w, j = columns[i].table;
            if(j==t[0]) w = r[0];
            else w = r[1];
            if(i>0) printf("%c",' ');
            tables[j][w][columns[i].attr].print();

        }
        printf("%c",'\n');
    }
}

void solve_sql()
{
    char csql[200];
    gets(csql);
    if(strlen(csql)>MAX_SQL_LEN) format_error("length of SQL > 100");

    vector<string> sql;
    word_split(csql, sql);

    //  locate SELECT, FROM and WHERE
    int p_select, p_from, p_where, p_END;
    locate(sql, p_select, p_from, p_where, p_END);
    //cerr << p_select << ' ' << p_from << ' ' << p_where << ' ' << p_END << endl;

    //  extract tables
    int t[2];
    extract_tables(sql, p_from, p_where, t);
    //cerr << t[0] << ' ' << t[1] << endl;

    vector<column_type> columns;
    extract_columns(sql, p_select, p_from, t, columns);
    /*
    cerr << columns.size() << endl;
    for(int i=0; i<columns.size(); i++)
    {
        cerr << columns[i].table << ' ' << columns[i].attr << ' ' << ta[columns[i].table].name[columns[i].attr] << endl;
    }
    */

    vector<cond_type1> cond_t[2];
    vector<cond_type2> cond_tt;
    extract_cond(sql, p_where, p_END, t, cond_t[0], cond_t[1], cond_tt);
    //cerr << "t[0]: " << cond_t[0].size() << endl;
    //cerr << "t[1]: " << cond_t[1].size() << endl;
    //cerr << "tt: " << cond_tt.size() << endl;

    vector<int> record_t[2];
    select_records(t[0], cond_t[0], record_t[0]);
    select_records(t[1], cond_t[1], record_t[1]);
    //cerr << t[0] << ' ' << record_t[0].size() << endl;
    //cerr << t[1] << ' ' << record_t[1].size() << endl;

    vector<pair<int, int>> record_tt;
    if(t[1]==-1)
    {
        for(int i=0; i<record_t[0].size(); i++)
            record_tt.push_back(pair<int, int>(record_t[0][i], -1));
    }
    else if(cond_tt.empty()){
        for(int i=0; i<record_t[0].size(); i++)
            for(int j=0; j<record_t[1].size(); j++)
                record_tt.push_back(pair<int, int>(record_t[0][i], record_t[1][j]));
    }
    else{
        int attr0, attr1;
        if(cond_tt[0].column1.table==t[0])
        {
            attr0 = cond_tt[0].column1.attr;
            attr1 = cond_tt[0].column2.attr;
        }
        else{
            attr0 = cond_tt[0].column2.attr;
            attr1 = cond_tt[0].column1.attr;
        }
        map<data, vector<int>> h;
        for(int j=0; j<record_t[1].size(); j++)
        {
            int r = record_t[1][j];
            h[tables[t[1]][r][attr1]].push_back(r);
        }
        for(int i=0; i<record_t[0].size(); i++)
        {
            int r = record_t[0][i];
            map<data, vector<int>>::iterator it = h.find(tables[t[0]][r][attr0]);
            if(it!=h.end())
            {
                for(int j=0; j<it->second.size(); j++)
                    record_tt.push_back(pair<int, int>(r, it->second[j]));
            }
        }
    }
    //cerr << record_tt.size() << endl;
    if(record_tt.size()>MAX_OUTPUT_SIZE) format_error("output size > 10000");

    if(columns.empty()) print_all_columns(record_tt, t);
    else print_columns(record_tt, t, columns);
}

int main()
{
    init_table();
    read_tables();
    int m;
    scanf("%d\n", &m);
    for(int i=0; i<m; i++)
        solve_sql();
    return 0;
}
