﻿{{ self.title() }}

{{ s('description') }}

顿顿想要建立一个简单的教务管理数据库，用于存储学生的考试成绩并支持一些基本的查询操作。

### 数据格式

该数据库包含以下五张**数据表**：

```
Student(sid, dept, age)
```
学生信息表：`sid` 为主键，表示学生 ID；`dept` 表示学生所在院系名称；`age` 表示学生的年龄。

```
Course(cid, name)
```
课程信息表：`cid` 为主键，表示课程 ID；`name` 表示课程名称。

```
Teacher(tid, dept, age)
```
教师信息表：`tid` 为主键，表示教师 ID；`dept` 表示教师所在院系名称；`age` 表示教师的年龄。

```
Grade(sid, cid, score)
```
成绩信息表：`sid` 和 `cid` 分别来自 `Student` 表和 `Course` 表的主键（即每条**记录**中的 `sid` 和 `cid` 一定存在于相应的**表**中，下同），二者一起作为该表的主键；`score` 表示该学生这门课程的成绩。


```
Teach(cid, tid)
```
授课信息表：`cid` 和 `tid` 分别来自 `Course` 表和 `Teacher` 表的主键，二者一起作为该表的主键；需要注意的是，一门课程可以有多个老师授课。

注：**主键（Primary Key）**可以唯一标识表中的每条记录，换言之在一张表中所有记录的主键互不相同。

在上述表中，`age` 和 `score` 是 $[0,100]$ 的整数，其余都是长度小于等于 $10$ 的非空字符串。其中三个主键 `sid`、`cid` 和 `tid` 由数字 `0...9` 组成，而 `name` 和 `dept` 则由大小写字母组成。

### 查询语句

该数据库支持使用一种简单的 `SELECT` 语句进行查询，文法定义如下所述。最终所有的查询语句都可以由 `QUERY` 符号推导而来。

在所有定义中，`::=` 表示其左侧的符号可以推导为右侧的符号串，其中带单引号的符号表示此符号为固定的字符串，不带引号的则表示符号还需要进行进一步推导。`[]` 表示文法中的可选内容。如果一个符号存在多种推导方式 `A ::= B` 和 `A ::= C`，则可以简写为 `A ::= B | C`。

```
QUERY ::= 'SELECT' '*' 'FROM' TABLES ['WHERE' EXPR]
QUERY ::= 'SELECT' COLUMNS 'FROM' TABLES ['WHERE' EXPR]
```
从指定的**表**（`TABLES`）中查询满足**筛选条件**（`EXPR`）的**记录**，并按照指定的**列**（`COLUMNS`）输出。如果没有给出**筛选条件**，则返回所有的**记录**。

```
TABLES ::= TABLE_NAME | TABLE_NAME ',' TABLE_NAME
TABLE_NAME ::= 'Student' | 'Grade' | 'Course' | 'Teach' | 'Teacher'
```

`TABLES` 可以包含多张**表**，此时新表中的**列**是这些表中列的笛卡尔积，一个例子如下所示。

```
+---------+ +---------+ +------------+ +------------+
| Table A | | Table B | | Table A, B | | Table B, A |
+---------+ +---------+ +------------+ +------------+
| a1      | | b1      | | a1, b1     | | b1, a1     |
| a2      | | b2      | | a1, b2     | | b1, a2     |
+---------+ | b3      | | a1, b3     | | b2, a1     |
            +---------+ | a2, b1     | | b2, a2     |
                        | a2, b2     | | b3, a1     |
                        | a2, b3     | | b3, a2     |
                        +------------+ +------------+
```

本题中约定 `TABLES` **仅会包含一张或两张不同的表**。

```
COLUMNS ::= COLUMN | COLUMNS ',' COLUMN
COLUMN ::= [TABLE_NAME '.'] COLUMN_NAME
COLUMN_NAME ::= 'sid' | 'cid' | 'tid' | 'age' | 'score' | 'dept' | 'name'
```

`COLUMNS` 也是递归定义，即由逗号分隔的至少一个 `COLUMN` 组成。在无歧义时（即 `TABLES` 中只有**唯一的表**含有该**列**），`COLUMN` 中的 `TABLE_NAME` 是可选的。`SELECT age FROM Student, Teacher` 是一种典型的歧义句式，因为无法确定其中想要选取的 `age` 是学生还是教师的年龄。

虽然在文法上一个 `COLUMN` 可以由任意 `TABLE_NAME` 和 `COLUMN_NAME` 组合而成，但在实际处理时，这两者需要符合上面定义的表结构。比如 `Course.sid` 就是一种错误的写法，因为 `Course` 表中并没有 `sid` 这一列。

```
EXPR ::= COND | EXPR 'AND' COND
COND ::= COLUMN CMP CONSTANT | COLUMN '=' COLUMN
CMP ::= '>' | '=' | '<'
```

其中：

* `EXPR` 是一个合取布尔表达式，由若干个**比较条件**（`COND`）用 `AND` 连接而成，表示**筛选条件**（即满足该条件的**记录**才会被输出）。
* `CONSTANT` 没有文法定义，它表示一个**常量**，且其类型和数据范围与相比较的 `COLUMN` 一致；如果是字符串类型，还需用双引号括起（例如 `"Sam"`）。

这里我们额外规定：

* **列**与**常量**进行比较时（`COLUMN CMP CONSTANT`），整数类型（`age`、`score`）可以进行大于、小于和等于三种判断，字符串类型只能做等于判断；
* 两**列**进行比较时（`COLUMN = COLUMN`），它们需来自不同的**表**且二者的**列名**（`COLUMN_NAME`）必须相同；
* 在一个 `EXPR` 中，`COLUMN CMP CONSTANT` 类型的 `COND` 个数不限，但 `COLUMN = COLUMN` 最多只能出现一次。

查询语句大小写敏感，全部的**保留字**包括：

```
SELECT, *, FROM, WHERE, AND, Student, Course, Teacher, Grade, Teach
sid, cid, tid, dept, name, age, score
```

全部的**分隔符**包括：`,`, `.`, `>`, `=`, `<`, ` `（空格）。

双引号（`"`）应视作字符串**常量**的一部分而非**保留字**或**分隔符**。

每个**保留字**和**常量**（`CONSTANT`）可视作查询语句中的一个**基本单位**，在格式上我们约定：

* 每条查询语句占一行；
* **基本单位**不可分割；
* 为避免歧义，两个**基本单位**之间应至少有一个**分隔符**，并且允许存在任意多的空格；
* 一行总长度不超过 $100$ 个字符。

### 结果输出

对每条查询输出一张**表**，**列**由 `COLUMNS` 指定，每行为一条满足**筛选条件**（`EXPR`）的**记录**（不同的**列**间用一个空格分隔）。这里只需要输出所有满足条件的**记录**，不需要打印表头；查询结果为空则不输出；**字符串类型无需输出双引号**。若想输出所有的**列**，可以用 `*` 代替 `COLUMNS`；若 `TABLES` 中只有一张**表**，此时应输出该**表**的全部**列**；如果有第二张**表**，应输出第一张**表**的全部**列**与二张**表**的全部**列**的笛卡尔积；表内部的**列**按照上文定义时给出的顺序排序。

如果 `TABLES` 仅含一张**表**，则按该**表**的顺序依次输出符合条件的**记录**。若 `TABLES` 包含两张**表**（形如 `A,B`），则先考虑 A 的顺序，再考虑 B 的顺序（参考前文中两张表做笛卡尔积的例子）。

{{ s('input format') }}

{{ self.input_file() }}

首先依次输入五张**表** `Student`、`Course`、`Teacher`、`Grade` 和 `Teach` 的数据。

对于第 $i$ 张**表**，第一行输入一个**正整数** $N_i$，表示该**表**中**记录**的个数（亦即行数）。接下来 $N_i$ 行，每行输入一条**记录**，其中不同**列**之间用一个空格分隔且字符串**字段**不会用双引号括起。

然后输入一个**正整数** $M$，表示需要处理的查询语句个数。最后 $M$ 行每行输入一条查询语句，保证每条查询语句均符合上述所有要求。

{{ s('output format') }}

{{ self.output_file() }}

按要求输出每条查询语句的结果。

{{ s('sample', 1) }}

{{ self.sample_text() }}

{{ self.title_sample_description() }}

第 $1-3$ 行、$4-6$ 行、$7-12$ 行和 $13-14$ 行依次对应四条查询的结果。

{{ s('sample', 2) }}

{{ self.sample_file() }}

{{ self.title_sample_description() }}

第 $1-2$ 行、$3-4$ 行、$5-6$ 行、$7-14$ 行、$15-16$ 行和 $17-18$ 行依次对应前六条查询的结果，最后一条查询结果为空因为不存在 ID 为 `2019` 的课程。

{{ s('subtasks') }}

本题共有六个子任务，每个子任务有一个或多个测试点。若你的程序对一个子任务的**全部**测试点，都能给出正确的输出，则得该子任务的满分，否则该子任务得 $0$ 分。本题满分共计 $100$ 分。

* 子任务一（${{ data[0].score }}$ 分）：所有测试点保证每条查询仅涉及**一张表**，且 $N_i \le {{ prob.args['subn'] }}$、$M \le {{ tools.hn(prob.args['subm']) }}$，查询语句形如 `SELECT * FROM TABLE_NAME`；
* 子任务二（${{ data[1].score }}$ 分）：所有测试点保证每条查询仅涉及**一张表**，且 $N_i \le {{ prob.args['subn'] }}$、$M \le {{ tools.hn(prob.args['subm']) }}$，查询语句形如 `SELECT * FROM TABLE_NAME WHERE EXPR`；
* 子任务三（${{ data[2].score }}$ 分）：所有测试点保证每条查询仅涉及**一张表**，且 $N_i \le {{ prob.args['subn'] }}$、$M \le {{ tools.hn(prob.args['subm']) }}$；
* 子任务四（${{ data[3].score }}$ 分）：所有测试点保证每条查询最多涉及**两张表**，每个 expr 中**均不存在** `COLUMN = COLUMN` 类型的 `COND`，且 $N_i \le {{ tools.hn(prob.args['n']) }}$、$M \le {{ tools.hn(prob.args['m']) }}$；
* 子任务五（${{ data[4].score }}$ 分）：所有测试点保证每条查询最多涉及**两张表**，每个 expr 中**最多一个** `COLUMN = COLUMN` 类型的 `COND`，且 $N_i \le {{ prob.args['subn'] }}$、$M \le {{ tools.hn(prob.args['m']) }}$；
* 子任务六（${{ data[5].score }}$ 分）：所有测试点保证每条查询最多涉及**两张表**，每个 expr 中**最多一个** `COLUMN = COLUMN` 类型的 `COND`，且 $N_i \le {{ tools.hn(prob.args['n']) }}$、$M \le {{ tools.hn(prob.args['m']) }}$。

此外，所有测试数据保证每条查询的结果均不超过 ${{ tools.hn(prob.args['n']) }}$ 行。

{{ s('hint') }}

在下发的文件中，我们还为选手提供了一些与真实测试数据生成方式相同的的输入文件，其中涵盖了实际测试中用到的**所有**查询语句。
